<?php
/**
 * Author        : Tengda
 * Date          : 2021/7/29
 * Time          : 11:12 上午
 */

use App\Base\Base;
use App\Services\goods\OtaService;
use \Validate\Ota\getBindListValidate;
use Validate\otaValidate;

class OtaController extends Base
{

    /**
     * 供应商列表
     */
    public function listAction()
    {
        (new otaValidate())->scene('list')->validate();

        $name = $this->params['ota_name'] ?? '';
        $printerSn = $this->params['printer_sn'] ?? '';
        $printerKey = $this->params['printer_key'] ?? '';
        $offset = $this->params['offset'] ?? 0;
        $limit = $this->params['limit'] ?? 20;
        $res = OtaService::list($name, $printerSn, $printerKey, $offset, $limit);

        $this->success(['data' => $res]);
    }

    /**
     * 添加供应商
     * @throws \App\Exception\custom\ParamException
     */
    public function addAction()
    {
        (new otaValidate())->scene('add')->validate();
        $res = OtaService::add($this->params);

        $this->success(['status' => $res ? 'success' : 'failed']);
    }

    /**
     * 删除一个供应商
     * @throws \App\Exception\custom\ParamException
     */
    public function deleteAction()
    {
        (new otaValidate())->scene('delete')->validate();
        $res = OtaService::deleteOne($this->params['ota_id']);

        $this->success(['status' => $res ? 'success' : 'failed']);
    }


    /**
     * 更新供应商
     * @throws Exception
     */
    public function updateAction()
    {
        (new otaValidate())->scene('update')->validate();
        $data = [];
        $this->params['ota_name'] && $data['ota_name'] = $this->params['ota_name'];
        $this->params['printer_sn'] && $data['printer_sn'] = $this->params['printer_sn'];
        $this->params['printer_key'] && $data['printer_key'] = $this->params['printer_key'];

        $res = $data ? OtaService::update($this->params['ota_id'], $data) : true;

        $this->success(['status' => $res ? 'success' : 'failed']);
    }
}