<?php


namespace App\Services\goods;


use Api\PhpServices\Idgen\Idgen;
use Api\PhpServices\Ksy\Ks3Api;
use Api\PhpServices\Ksy\Ksyun;
use Api\PhpServices\Sensitive\Sensitive;
use Api\PhpUtils\Common\BaseConvert;
use Api\PhpUtils\Common\GoodsSkuId;
use Api\PhpUtils\Http\HttpUtil;
use App\Exception\custom\GoodsException;
use App\Models\goods\mysql\Category;
use App\Models\goods\mysql\GoodsOperationRecord;
use App\Models\goods\mysql\GoodsSku;
use App\Models\goods\mysql\GoodsSkuPicRecord;
use App\Models\goods\mysql\GoodsSkuSubShop;
use App\Models\goods\mysql\GoodsSnapshot;
use App\Models\goods\mysql\GoodsSpu;
use App\Models\goods\mysql\LifeAccountShopNum;
use App\Models\goods\mysql\PaySuccessGoodsCallbackRecord;
use App\Models\goods\mysql\PindanGoodsSku;
use App\Models\goods\mysql\Shop;
use App\Models\marketing\mysql\Distributor;
use App\Models\marketing\mysql\Marketing;
use App\Models\marketing\mysql\MarketingGoods;
use App\Models\shop\mysql\SubShop;
use App\Services\marketing\DistributorService;
use App\Services\marketing\MarketingGoodsService;
use App\Services\marketing\MarketingService;
use App\Services\shop\ShopService;
use App\Services\common\CommonService;

use Api\PhpUtils\Http\Request;
use App\Exception\custom\InterfaceException;

class MarketingPindanGoodsService
{
    /**
     * 拼单商品
     * @param $params
     * @return string
     * @throws GoodsException
     * @throws \App\Exception\custom\CodeSpecialException
     */
    public static function pindanMarketingAddGoods($params)
    {
        GoodsSpu::beginTransaction();

        //生活号信息
        $params["life_account_id"] = MarketingService::PUBLIC_LIFE_ACCOUNT_ID;

        $shopId = "00";
        $spuData = self::addGoodsSpu($shopId, $params);
        $skuId = self::addGoodsSku($spuData, $shopId, $params);
        //商品操作记录
        $record = [
            "goods_spu_id" => $spuData["goods_spu_id"],
            "goods_sku_id" => $skuId,
            "operator_id" => empty($params["publish_user_id"]) ? 0 : $params["publish_user_id"],
            "operator_name" => $params["user_name"],
            "goods_status" => GoodsSku::STATUS_PASS,
            "online_status" => GoodsSku::ONLINE_STATUS_ONLINE,
            "before_version" => 1,
            "after_version" => 1,
            "note" => empty($params["rejected_reason"]) ? "" : $params["rejected_reason"],
        ];
        GoodsOperationRecord::insertRecord($record);

        if (!GoodsSpu::commit()) {
            GoodsSpu::rollback();
            throw new GoodsException(["cus" => 0]);
        }
        return $skuId;
    }

    /**
     * 添加spu
     * @param $shopId
     * @param array $spuData
     * @return mixed
     * @throws GoodsException
     */
    private static function addGoodsSpu($shopId, $spuData = [])
    {
        $spu = GoodsSpu::get("*", ["goods_name" => $spuData["marketing_name"], "life_account_id" => $spuData["life_account_id"]]);
        if (!empty($spu)) {
            return $spu;
        }

        $res = GoodsService::getIdgenId(substr($shopId, -2), "goods");
        if (empty($res)) {
            throw new GoodsException(['cus' => 2]);
        }
        $spuId = $res[0];

        $spuParams = [
            "goods_spu_id" => $spuId,
            "shop_id" => $shopId,
            "life_account_id" => $spuData["life_account_id"],
            "publish_user_id" => $spuData["publish_user_id"],
            "goods_name" => $spuData["marketing_name"],
            "desc_pic_url" => $spuData["desc_pic_url"],
            "desc" => $spuData["desc"],
            "goods_type" => empty($spuData["goods_type"]) ? GoodsSpu::TYPE_VIRTUAL : $spuData["goods_type"],
            "goods_sub_type" => empty($spuData["goods_sub_type"]) ? GoodsSpu::SUB_TYPE_TO_SHOP : $spuData["goods_sub_type"],
            "belong_type" => empty($spuData["belong_type"]) ? GoodsSpu::BELONG_TYPE_DISANFANG : $spuData["belong_type"],
            "goods_sale_type" => empty($spuData["setmeal"]) ? GoodsSpu::SALE_TYPE_NO_COMBINATION : GoodsSpu::SALE_TYPE_COMBINATION,
            "rule_limit" => $spuData["rule_limit"],
            "rule_desc" => $spuData["rule_desc"],
            "rule_refund" => $spuData["rule_refund"],
        ];
        GoodsSpu::save($spuParams);
        return $spuParams;
    }

    /**
     * 添加拼单活动sku
     * @param $spuData
     * @param $shopId
     * @param array $skuData
     * @return string
     * @throws GoodsException
     * @throws \App\Exception\custom\CodeSpecialException
     */
    private static function addGoodsSku($spuData, $shopId, $skuData = [])
    {
        $pindanSku = PindanGoodsSku::get("pindan_goods_sku_id", ["goods_name" => $skuData["goods_name"]
            , "life_account_id" => $skuData["life_account_id"]]);
        if (!empty($pindanSku)) {
            throw new GoodsException(['cus' => 17]);
        }

        $res = GoodsService::getIdgenId(substr($shopId, -2), "goods");
        if (empty($res)) {
            throw new GoodsException(['cus' => 2]);
        }
        $skuId = GoodsSkuId::generateGoodSkuId($res[0], 0, 0, $spuData["goods_type"], $spuData["belong_type"]);

        if ($skuData["price"] <= 0) {
            throw new GoodsException(['cus' => 33]);
        }

        if ($skuData["price"] > 50000) {
            throw new GoodsException(['cus' => 34]);
        }
        $checkGoodsName = GoodsService::isHaveSensitive($skuData["goods_name"], 1);
        if ($checkGoodsName) {
            throw new GoodsException(['cus' => 19]);
        }
        $checkGoodsDesc = GoodsService::isHaveSensitive($skuData["desc"], 2);
        if ($checkGoodsDesc) {
            throw new GoodsException(['cus' => 20]);
        }
        $skuParams = [
            "pindan_goods_sku_id" => $skuId,
            "goods_spu_id" => $spuData["goods_spu_id"],
            "life_account_id" => $skuData["life_account_id"],
            "publish_user_id" => empty($skuData["publish_user_id"]) ? "" : $skuData["publish_user_id"],
            "ota_id" => $skuData["ota_id"],
            "category_1_id" => 0,
            "category_2_id" => 0,
            "goods_name" => $skuData["goods_name"],
            "desc_pic_url" => $skuData["desc_pic_url"],
            "desc" => $skuData["desc"],
            "inventory_total" => $skuData["inventory"],
            "inventory_rest" => $skuData["inventory"],
            "original_price" => empty($skuData["original_price"]) ? "" : $skuData["original_price"] * 100,
            "price" => $skuData["price"] * 100,
            "audit_status" => PindanGoodsSku::STATUS_PASS,
            "online_status" => PindanGoodsSku::ONLINE_STATUS_ONLINE,
        ];

        $res = PindanGoodsSku::save($skuParams);
        if ($res === false) {
            GoodsSpu::rollback();
            throw new GoodsException(["cus" => 0]);
        }
        GoodsService::addGoodsSkuPicRecord($skuId, $skuData);
        return $skuId;
    }

    /**
     * 编辑拼单商品
     * @param $params
     * @return bool
     * @throws GoodsException
     * @throws \App\Exception\custom\CodeSpecialException
     */
    public static function pindanMarketingEditGoods($params)
    {
        $goodsSkuId = $params["pindan_goods_sku_id"];
        $pindanGoodsSku = PindanGoodsSku::get("*", ["pindan_goods_sku_id" => $goodsSkuId]);
        if (empty($pindanGoodsSku)) {
            throw new GoodsException(["cus" => 15]);
        }

        GoodsSpu::beginTransaction();

        $goodsSpu = GoodsSpu::getRecord(["goods_spu_id" => $pindanGoodsSku["goods_spu_id"]]);

        self::editGoodsSpu($goodsSpu, $params);
        self::editGoodsSku($goodsSkuId, $pindanGoodsSku, $params);

        if (!GoodsSpu::commit()) {
            GoodsSpu::rollback();
            throw new GoodsException(["cus" => 0]);
        }

        return true;
    }

    /**
     * 编辑拼单spu
     * @param $goodsSpu
     * @param array $params
     * @return \Api\PhpUtils\Mysql\MysqlBase
     * @throws GoodsException
     */
    private static function editGoodsSpu($goodsSpu, $params = [])
    {
        $checkName = GoodsService::checkGoodsName($params["goods_name"], $goodsSpu["life_account_id"], $goodsSpu["goods_spu_id"]);
        if (!$checkName["can_use"]) {
            throw new GoodsException(['cus' => 17]);
        }

        return GoodsSpu::save([
            "desc_pic_url" => $params["desc_pic_url"],
            "desc" => $params["desc"],
        ], ["goods_spu_id" => $goodsSpu["goods_spu_id"]]);
    }

    /**
     * 编辑拼单sku
     * @param $goodsSkuId
     * @param $skuData
     * @param array $params
     * @return \Api\PhpUtils\Mysql\MysqlBase
     * @throws GoodsException
     * @throws \App\Exception\custom\CodeSpecialException
     */
    private static function editGoodsSku($goodsSkuId, $skuData, $params = [])
    {
        $inventoryCount = $skuData["inventory_rest"] + $params["inventory_add"];
        //$inventoryCount = $params["inventory"];
        if ($inventoryCount < 0) {
            throw new GoodsException(['cus' => 18]);
        }
        $checkGoodsName = GoodsService::isHaveSensitive($params["goods_name"], 1);
        if ($checkGoodsName) {
            throw new GoodsException(['cus' => 19]);
        }
        $checkGoodsDesc = GoodsService::isHaveSensitive($params["desc"], 2);
        if ($checkGoodsDesc) {
            throw new GoodsException(['cus' => 20]);
        }

        if ($params["price"] <= 0) {
            throw new GoodsException(['cus' => 33]);
        }

        if ($params["price"] > 50000) {
            throw new GoodsException(['cus' => 34]);
        }


        $skuParams = [
            "category_1_id" => $params["category_1_id"],
            "category_2_id" => $params["category_2_id"],
            "goods_name" => $params["goods_name"],
            "desc_pic_url" => $params["desc_pic_url"],
            "desc" => $params["desc"],
            "inventory_total" => $skuData["inventory_total"] + $params["inventory_add"],
            //"inventory_total" => $params["inventory"],
            "inventory_rest" => $inventoryCount,
            "original_price" => empty($params["original_price"]) ? "" : $params["original_price"] * 100,
            "price" => $params["price"] * 100,
            "goods_version" => (int)$skuData["goods_version"] + 1,
        ];

        //商品操作记录
        $record = [
            "goods_spu_id" => $skuData["goods_spu_id"],
            "goods_sku_id" => $goodsSkuId,
            "operator_id" => empty($params["publish_user_id"]) ? 0 : $params["publish_user_id"],//op后台操作没有记录user_id
            "operator_name" => $params["user_name"],
            "goods_status" => GoodsSku::STATUS_PASS,
            "online_status" => GoodsSku::ONLINE_STATUS_ONLINE,
            "before_version" => $skuData["goods_version"],
            "after_version" => $skuParams["goods_version"],
            "note" => empty($params["rejected_reason"]) ? "" : $params["rejected_reason"],
        ];
        GoodsOperationRecord::insertRecord($record);

        GoodsService::addGoodsSkuPicRecord($goodsSkuId, $skuParams);

        return PindanGoodsSku::save($skuParams, ["pindan_goods_sku_id" => $goodsSkuId]);
    }

}