<?php


namespace App\Services\marketing;

use App\Models\goods\mysql\PindanGoodsSku;
use App\Models\marketing\mysql\Marketing;
use App\Models\marketing\mysql\MarketingGoods;
use App\Models\goods\mysql\GoodsSku;
use Api\PhpServices\Idgen\Idgen;
use App\Exception\custom\MarketingException;
use Api\PhpServices\Sensitive\Sensitive;
use App\Models\marketing\mysql\MarketingPindan;
use App\Services\common\CommonService;
use Daemon\Goods;


class MarketingService
{
    const PUBLIC_LIFE_ACCOUNT_ID = 111;

    /**
     * op 后台获取活动需要参加的商品
     *
     * @param $params
     * @return array|mixed
     */
    public static function marketingGooodsList($params='')
    {
        $startTime = !empty($params['start_time']) ? $params['start_time'] : '';
        $endTime = !empty($params['end_time']) ? $params['end_time'] : '';
        $goodsSpuId = !empty($params['goods_spu_id']) ? $params['goods_spu_id'] : '';
        $lifeAccountId = !empty($params['life_account_id']) ? $params['life_account_id'] : '';

        if (empty($startTime)) {
             throw new MarketingException(['cus'=>8]);
        }

        if (empty($endTime)) {
            throw new MarketingException(['cus'=>9]);
        }

        // if ($startTime <= date("Y-m-d")) {
        //      throw new MarketingException(['cus'=>10]);
        // }

        if ($endTime <= date("Y-m-d")) {
            throw new MarketingException(['cus'=>11]);
        }

        if ($startTime >= $endTime) {
            throw new MarketingException(['cus'=>14]);
        }
        
        $goodsData = [
            "OR" => [
                "start_time[>=]" => $startTime,
                "end_time[>=]" => $startTime,
            ],
            'online_status' => 1,
        ];

        //已存在的活动
        $tmpMarketingList = Marketing::getMarketingList($goodsData);
        $tmpMarketingId = [];
        if (!empty($tmpMarketingList)) {
            $tmpMarketingId = array_column($tmpMarketingList, 'marketing_id');
        }
        //已加入活动的商品
        $marketingGoodsList = MarketingGoods::marketingGoodsList(['marketing_id' => $tmpMarketingId]);
        if (!empty($marketingGoodsList)) {
            $tmpGoodsSkuId = array_column($marketingGoodsList, 'goods_sku_id');
        }


        $where = [
            'online_status' => 1,
            'rule_refund' => 2,
            "expiration_time[>]" => date("Y-m-d"),
        ];

        if (!empty($tmpGoodsSkuId)) {
            $where['goods_sku_id[!]'] = $tmpGoodsSkuId;
        }

        if (!empty($goodsSpuId)) {
            $where['goods_spu_id'] = $goodsSpuId;
        }

        if (!empty($lifeAccountId)) {
            $where['life_account_id'] = $lifeAccountId;
        }
        
        $goodsSkuList = GoodsSku::select('*', $where);

        $lifeAccountIds = [];
        if (!empty($goodsSkuList)) {
            $lifeAccountIds = array_column($goodsSkuList, 'life_account_id');
        }
        $lifeAccountList = CommonService::getlifeAccountList(['life_account_id' => $lifeAccountIds]);
        $list = [];
        foreach ($goodsSkuList as $key => $value) {
            $list[$key]['goods_sku_id'] = $value['goods_sku_id'];
            $list[$key]['goods_spu_id'] = $value['goods_spu_id'];
            $list[$key]['life_account_id'] = $value['life_account_id'];
            $list[$key]['goods_name'] = $value['goods_name'];
            $list[$key]['life_account_name'] = !empty($lifeAccountList[$value['life_account_id']]['life_account_name']) ? $lifeAccountList[$value['life_account_id']]['life_account_name'] : '';  
        }
        return $list;
    }

    /**
     * op 后台列表查询
     *
     * @param $params
     * @return array|mixed
     */
    public static function marketingList($params)
    {
        $marketingType = !empty($params['marketing_type']) ? $params['marketing_type'] : Marketing::MARKETING_TYPE_FENXIAO;
        if ($marketingType == Marketing::MARKETING_TYPE_FENXIAO) {
            return self::fenxiaoMarketingList($params);
        } else if ($marketingType == Marketing::MARKETING_TYPE_TUANGOU) {
            return [];
        } else if ($marketingType == Marketing::MARKETING_TYPE_MIAOSHA) {
            return [];
        } else if ($marketingType == Marketing::MARKETING_TYPE_PINDAN) {
            return self::pindanMarketingList($params);
        }
    }

    /**
     * 分销活动列表
     * @param $params
     * @return array
     */
    private static function fenxiaoMarketingList($params)
    {
        $params['page'] = !empty($params['page']) ? $params['page'] : 1;
        $limit = !empty($params['page_size']) ? $params['page_size'] : 20;
        $page = ($params['page'] - 1) * $limit;

        $where["marketing_type"] = Marketing::MARKETING_TYPE_FENXIAO;
        if (!empty($params['marketing_name'])) {
            $where['marketing_name'] = $params['marketing_name'];
        }

        if (!empty($params['marketing_id'])) {
            $where['marketing_id'] = $params['marketing_id'];
        }

        if (!empty($params['online_status'])) {
            if (is_array($params['online_status'])) {
                foreach ($params['online_status'] as $key => $valu) {
                    if (!empty($valu)) {
                        $where['online_status'][$key] = $valu;
                    }else{
                        $where['online_status'] = [1,2];
                        break;
                    }
                }
            }else{
                $where['online_status'] = $params['online_status'];
            }
        }

        if (!empty($params['goods_spu_id'])) {
            $goodsSpuId = $params['goods_spu_id'];
            $marketingGoodsList = MarketingGoods::marketingGoodsList(['goods_spu_id' => $goodsSpuId]);

            if (empty($marketingGoodsList)) {
                $where['marketing_id'] = "-1";
            }else{
                $marketingId = array_column($marketingGoodsList, 'marketing_id');
                if (!empty($where['marketing_id'])) {
                    array_push($marketingId,$where['marketing_id']);
                }
                $where['marketing_id'] = $marketingId;
            }

        }

        $where['ORDER'] = ["create_time" => "DESC"];
        $where['LIMIT'] = [$page, $limit];
        $list = Marketing::getRecords($where);
        $lists = [];
        foreach ($list as $key => $value) {
            $lists[$key] = $value;
            $lists[$key]['first_commission_value'] = !empty($value['first_commission_value']) ? $value['first_commission_value']/100 : 0;
            $lists[$key]['second_commission_value'] = !empty($value['second_commission_value']) ? $value['second_commission_value']/100 : 0;
        }
        unset($where['LIMIT']);
        unset($where['ORDER']);
        $count = Marketing::getCount($where);
        $data = ['result' => $lists,'count' => $count];
        return $data;
    }

    /**
     * 拼单活动列表
     * @param $params
     * @return array
     */
    private static function pindanMarketingList($params)
    {
        $params['page'] = !empty($params['page']) ? $params['page'] : 1;
        $limit = !empty($params['page_size']) ? $params['page_size'] : 20;
        $page = ($params['page'] - 1) * $limit;

        $where["marketing_type"] = Marketing::MARKETING_TYPE_PINDAN;
        if (!empty($params['marketing_name'])) {
            $where['marketing_name[~]'] = $params['marketing_name'];
        }
        if (!empty($params['online_status'])) {
            $where['online_status'] = $params['online_status'];
        }

        $where['ORDER'] = ["create_time" => "DESC"];
        $where['LIMIT'] = [$page, $limit];
        $list = Marketing::select(["marketing_id", "marketing_name", "start_time", "end_time",
                "online_status", "update_time", "create_time"]
            , $where);

        $marketingIds = [];
        foreach ($list as $key => $value) {
            $marketingIds[] = $value["marketing_id"];
        }
        $marketingPindanData = MarketingPindan::select(["marketing_id", "min_price", "max_price", "pindan_desc"]
            , ["marketing_id" => $marketingIds]);
        $priceData = [];
        if (!empty($marketingPindanData)) {
            foreach ($marketingPindanData as $item) {
                $priceData[$item["marketing_id"]] = $item;
            }
        }

        $lists = [];
        foreach ($list as $key => $value) {
            $lists[$key] = $value;
            $lists[$key]["online_status_desc"] = Marketing::$onlineStatusDesc[$value["online_status"]];
            if (!empty($priceData[$value["marketing_id"]])) {
                $lists[$key]["min_price"] = $priceData[$value["marketing_id"]]["min_price"];
                $lists[$key]["max_price"] = $priceData[$value["marketing_id"]]["max_price"];
                $lists[$key]["pindan_desc"] = $priceData[$value["marketing_id"]]["pindan_desc"];
            }
        }
        unset($where['LIMIT']);
        unset($where['ORDER']);
        $count = Marketing::count("*", $where);
        return ['result' => $lists,'count' => $count];
    }

    /**
     * 创建活动
     * @param $params
     * @return \Api\PhpUtils\Mysql\MysqlBase|array
     * @throws MarketingException
     * @throws \App\Exception\custom\CodeSpecialException
     */
    public static function addMarketing($params)
    {
        $marketingType = !empty($params['marketing_type']) ? $params['marketing_type'] : Marketing::MARKETING_TYPE_FENXIAO;
        if ($marketingType == Marketing::MARKETING_TYPE_FENXIAO) {
            return self::addFenxiaoMarketing($params);
        } else if ($marketingType == Marketing::MARKETING_TYPE_TUANGOU) {
            return [];
        } else if ($marketingType == Marketing::MARKETING_TYPE_MIAOSHA) {
            return [];
        } else if ($marketingType == Marketing::MARKETING_TYPE_PINDAN) {
            return self::addPindanMarketing($params);
        }
    }

    /**
     * 增加分销活动
     * @param $params
     * @return \Api\PhpUtils\Mysql\MysqlBase
     * @throws MarketingException
     * @throws \App\Exception\custom\CodeSpecialException
     */
    private static function addFenxiaoMarketing($params)
    {
        $marketingName = !empty($params['marketing_name']) ? $params['marketing_name'] : '';
        $firstCommissionRate = !empty($params['first_commission_value']) ? $params['first_commission_value']*100 : 0;
        $secondCommissionRate = !empty($params['second_commission_value']) ? $params['second_commission_value']*100 : 0;
        $createUserEmail = !empty($params['op_cur_user']) ? $params['op_cur_user'] : '';
        $startTime = !empty($params['start_time']) ? $params['start_time'] : '';
        $endTime = !empty($params['end_time']) ? $params['end_time'] : '';
        $type = !empty($params['marketing_type']) ? $params['marketing_type'] : 1;
        $goodsSkuId = !empty($params['goods_sku_id']) ? explode(",", $params['goods_sku_id']) : [];

        $rate = $firstCommissionRate + $secondCommissionRate;
        if ($rate > 5000) {
            throw new MarketingException(['cus'=>15]);
        }

        if (empty($goodsSkuId)) {
            throw new MarketingException(['cus'=>12]);
        }

        if (empty($marketingName)) {
            throw new MarketingException(['cus'=>0]);
        }

        // if (empty($firstCommissionRate)) {
        //     throw new MarketingException(['cus'=>3]);
        // }

        if (empty($secondCommissionRate)) {
            throw new MarketingException(['cus'=>4]);
        }

        if (self::utf8Strlen($marketingName) > 50) {
            throw new MarketingException(['cus'=>1]);
        }

        if (empty($startTime)) {
            throw new MarketingException(['cus'=>8]);
        }

        if (empty($endTime)) {
            throw new MarketingException(['cus'=>9]);
        }

        // if ($startTime <= date("Y-m-d")) {
        //      throw new MarketingException(['cus'=>10]);
        // }

        if ($endTime <= date("Y-m-d")) {
            throw new MarketingException(['cus'=>11]);
        }

        if ($startTime >= $endTime) {
            throw new MarketingException(['cus'=>14]);
        }

        $tmpMarketingId = [];
        $tmpGoodsSkuId = [];
        $goodsData = [
            "OR" => [
                "start_time[>=]" => $startTime,
                "end_time[>=]" => $startTime,
            ],
            "online_status" => 1,
        ];

        //存在的活动
        $tmpMarketingList = Marketing::getMarketingList($goodsData);

        if (!empty($tmpMarketingList)) {
            $tmpMarketingId = array_column($tmpMarketingList, 'marketing_id');
        }

        $marketingGoodsList = MarketingGoods::marketingGoodsList(['marketing_id' => $tmpMarketingId]);

        if (!empty($marketingGoodsList)) {
            $tmpGoodsSkuId = array_column($marketingGoodsList, 'goods_sku_id');
        }

        if (!empty($tmpGoodsSkuId)) {
            if (array_intersect($goodsSkuId, $tmpGoodsSkuId)) {
                throw new MarketingException(['cus'=>13]);
            }
        }


        $detectParams = [
            'businessId' => 1,
            'text' => !empty($marketingName) ? $marketingName : '',
            'scene' => 1,
        ];
        $sensitive =  (new Sensitive)->detect($detectParams);
        if (!empty($sensitive['data'])) {
            throw new MarketingException(['cus'=>2]);
        }

        // //判断重复
        // if (!self::isName($name)) {
        //     throw new ShopException(['cus'=>6]);
        // }
        //$res = self::getIdgenId(1, 'goods', $count = 1);
        //$stores_id = !empty($res[0]) ? $res[0] : '';

        Marketing::beginTransaction();
        $colums = [
            //'marketing_id' => $marketing_id,
            'marketing_name' => $marketingName,
            'first_commission_value' => $firstCommissionRate,
            'second_commission_value' => $secondCommissionRate,
            'create_user_email' => $createUserEmail,
            'good_count' => 0,
            'start_time' => $startTime,
            'end_time' => $endTime,
            'online_status' => 1,
            'marketing_type' => $type,
            'update_user_email' => $createUserEmail,
            'update_time' => date("Y-m-d H:i:s"),
            'create_time' => date("Y-m-d H:i:s"),
        ];
        $marketingId = Marketing::insertRecord($colums);
        if (empty($marketingId)) {
            Marketing::rollback();
            throw new MarketingException(['cus' => 5]);
        }
        $colums = [];
        $goodCount = !empty($goodsSkuId) ? count($goodsSkuId) : 0;
        $goodsSkuList = GoodsSku::select('*', ['goods_sku_id' => $goodsSkuId]);
        foreach ($goodsSkuList as $key => $value) {
            $colums[$key]['marketing_id'] = $marketingId;
            $colums[$key]['goods_spu_id'] = $value['goods_spu_id'];
            $colums[$key]['goods_sku_id'] = $value['goods_sku_id'];
            $colums[$key]['update_time'] = date("Y-m-d H:i:s");
            $colums[$key]['create_time'] = date("Y-m-d H:i:s");
        }

        $info = MarketingGoods::insertRecord($colums);
        if (empty($info)) {
            Marketing::rollback();
            throw new MarketingException(['cus' => 5]);
        }

        $srt = Marketing::updateRecord(['good_count' => $goodCount], ['marketing_id' => $marketingId]);
        if (!empty($srt)) {
            Marketing::commit();
            return $srt;
        }else{
            Marketing::rollback();
            throw new MarketingException(['cus' => 5]);
        }
    }

    /**
     * 增加拼单活动
     * @param $params
     * @return \Api\PhpUtils\Mysql\MysqlBase
     * @throws MarketingException
     */
    private static function addPindanMarketing($params)
    {
        $marketingName = !empty($params['marketing_name']) ? $params['marketing_name'] : '';
        $pindanPic = !empty($params['pindan_pic']) ? $params['pindan_pic'] : '';
        $pindanDesc = !empty($params['pindan_desc']) ? $params['pindan_desc'] : '';
        $createUserEmail = !empty($params['op_cur_user']) ? $params['op_cur_user'] : '';
        $startTime = !empty($params['start_time']) ? $params['start_time'] : '';
        $endTime = !empty($params['end_time']) ? $params['end_time'] : '';
        $type = Marketing::MARKETING_TYPE_PINDAN;
        $goodsSkuId = !empty($params['goods_sku_id']) ? explode(",", $params['goods_sku_id']) : [];
        $publicLifeAccountId = self::PUBLIC_LIFE_ACCOUNT_ID;

        //重复判断
        $marketing = Marketing::getRecord(["marketing_name" => $marketingName], ["marketing_id"]);
        if (!empty($marketing)) {
            $marketingPindan = MarketingPindan::getRecord(["marketing_id" => $marketing["marketing_id"]]);
            if (!empty($marketingPindan) && $publicLifeAccountId == $marketingPindan["publish_life_account_id"]) {
                throw new MarketingException(['cus' => 18]);
            }
        }

        if (empty($goodsSkuId)) {
            throw new MarketingException(['cus' => 12]);
        }

        //获取商品的最大最小值
        $skuInfoList = PindanGoodsSku::getRecordMaster(["pindan_goods_sku_id" => $goodsSkuId]
            , ["pindan_goods_sku_id", "goods_spu_id", "price"]);
        $priceList = [];
        foreach ($skuInfoList as $skuInfo) {
            $priceList[] = $skuInfo["price"];
        }
        $uniquePriceList = array_unique($priceList);
        sort($uniquePriceList);
        $minPrice = $priceList[0];
        $maxPrice = end($priceList);

        Marketing::beginTransaction();
        $marketingParams = [
            'marketing_name' => $marketingName,
            'start_time' => $startTime,
            'end_time' => $endTime,
            'online_status' => 1,
            'marketing_type' => $type,
            'update_user_email' => $createUserEmail,
            'update_time' => date("Y-m-d H:i:s"),
            'create_time' => date("Y-m-d H:i:s"),
        ];
        $marketingId = Marketing::insertRecord($marketingParams);
        if (empty($marketingId)) {
            Marketing::rollback();
            throw new MarketingException(['cus' => 5]);
        }

        $pindanParams = [
            "marketing_id" => $marketingId,
            "publish_life_account_id" => $publicLifeAccountId,
            "pindan_pic" => $pindanPic,
            "pindan_desc" => $pindanDesc,
            "min_price" => $minPrice,
            "max_price" => $maxPrice,
        ];
        $pindanId = MarketingPindan::insertRecord($pindanParams);
        if (empty($pindanId)) {
            Marketing::rollback();
            throw new MarketingException(['cus' => 5]);
        }

        $marketingGoods = [];
        foreach ($skuInfoList as $key => $skuInfo) {
            $marketingGoods[$key]["goods_spu_id"] = $skuInfo["goods_spu_id"];
            $marketingGoods[$key]["goods_sku_id"] = $skuInfo["pindan_goods_sku_id"];
            $marketingGoods[$key]["marketing_id"] = $marketingId;
        }
        MarketingGoods::save($marketingGoods);

        if (!Marketing::commit()) {
            Marketing::rollback();
            throw new MarketingException(["cus" => 5]);
        }

        return $pindanId;
    }

    /**
     * 更新数据
     *
     */
    public static function updateMarketing($params, $where, $originParams)
    {
        $marketingType = !empty($originParams['marketing_type']) ? $originParams['marketing_type'] : Marketing::MARKETING_TYPE_FENXIAO;
        if ($marketingType == Marketing::MARKETING_TYPE_FENXIAO) {
            $where['marketing_id'] = !empty($where['marketing_id']) ? $where['marketing_id'] : '';
            return Marketing::updateRecord($params, $where);
        } else if ($marketingType == Marketing::MARKETING_TYPE_TUANGOU) {
            return [];
        } else if ($marketingType == Marketing::MARKETING_TYPE_MIAOSHA) {
            return [];
        } else if ($marketingType == Marketing::MARKETING_TYPE_PINDAN) {
            return self::updatePindanMarketing($originParams);
        }
    }

    /**
     * 增加拼单活动
     * @param $params
     * @return \Api\PhpUtils\Mysql\MysqlBase
     * @throws MarketingException
     */
    private static function updatePindanMarketing($params)
    {
        $marketingId = $params["marketing_id"];
        //重复判断
        $marketing = Marketing::getRecord(["marketing_id" => $marketingId]);
        if (empty($marketing)) {
            throw new MarketingException(['cus' => 19]);
        }
        $marketingPindan = MarketingPindan::getRecord(["marketing_id" => $marketingId]);
        if (empty($marketingPindan)) {
            throw new MarketingException(['cus' => 19]);
        }

        $marketingName = !empty($params['marketing_name']) ? $params['marketing_name'] : '';
        $pindanPic = !empty($params['pindan_pic']) ? $params['pindan_pic'] : '';
        $pindanDesc = !empty($params['pindan_desc']) ? $params['pindan_desc'] : '';
        $createUserEmail = !empty($params['op_cur_user']) ? $params['op_cur_user'] : '';
        $startTime = !empty($params['start_time']) ? $params['start_time'] : '';
        $endTime = !empty($params['end_time']) ? $params['end_time'] : '';
        $type = Marketing::MARKETING_TYPE_PINDAN;
        $goodsSkuId = !empty($params['goods_sku_id']) ? explode(",", $params['goods_sku_id']) : [];
        if (empty($goodsSkuId)) {
            throw new MarketingException(['cus' => 12]);
        }

        //获取商品的最大最小值
        $skuInfoList = PindanGoodsSku::getRecordMaster(["pindan_goods_sku_id" => $goodsSkuId]
            , ["pindan_goods_sku_id", "goods_spu_id", "price"]);
        $priceList = [];
        foreach ($skuInfoList as $skuInfo) {
            $priceList[] = $skuInfo["price"];
        }
        $uniquePriceList = array_unique($priceList);
        sort($uniquePriceList);
        $minPrice = $priceList[0];
        $maxPrice = end($priceList);

        Marketing::beginTransaction();
        $marketingParams = [
            'marketing_name' => $marketingName,
            'start_time' => $startTime,
            'end_time' => $endTime,
            'online_status' => 1,
            'marketing_type' => $type,
            'update_user_email' => $createUserEmail,
            'update_time' => date("Y-m-d H:i:s"),
            'create_time' => date("Y-m-d H:i:s"),
        ];
        Marketing::updateRecord($marketingParams, ["marketing_id" => $marketingId]);

        $pindanParams = [
            "pindan_pic" => $pindanPic,
            "pindan_desc" => $pindanDesc,
            "min_price" => $minPrice,
            "max_price" => $maxPrice,
        ];
        MarketingPindan::updateRecord($pindanParams, ["marketing_id" => $marketingId]);

        $marketingGoods = MarketingGoods::getRecords(["marketing_id" => $marketingId]);
        if (!empty($marketingGoods)) {
            $ids = array_column($marketingGoods, "id");
            $delRes = MarketingGoods::delete(["id"=>$ids]);
            if ($delRes == 0) {
                throw new MarketingException(['cus' => 7]);
            }
        }
        $marketingGoods = [];
        foreach ($skuInfoList as $key => $skuInfo) {
            $marketingGoods[$key]["goods_spu_id"] = $skuInfo["goods_spu_id"];
            $marketingGoods[$key]["goods_sku_id"] = $skuInfo["pindan_goods_sku_id"];
            $marketingGoods[$key]["marketing_id"] = $marketingId;
        }
        MarketingGoods::save($marketingGoods);

        if (!Marketing::commit()) {
            Marketing::rollback();
            throw new MarketingException(["cus" => 5]);
        }

        return true;
    }


    public static function marketingInfo($params)
    {
        $where['marketing_id'] = !empty($params['marketing_id']) ? $params['marketing_id'] : '';
        $marketing_info = Marketing::getRecord($where);

        $goodsSkuList = MarketingGoods::marketingGoodsList($where);

        $goodsSkuId = array_column($goodsSkuList , 'goods_sku_id');

 
        $goodsSkuList = GoodsSku::select('*', ['goods_sku_id' => $goodsSkuId]);

        $lifeAccountIds = [];
        if (!empty($goodsSkuList)) {
            $lifeAccountIds = array_column($goodsSkuList, 'life_account_id');
        }
        $lifeAccountList = CommonService::getlifeAccountList(['life_account_id' => $lifeAccountIds]);
        $list = [];

        if (!empty($goodsSkuList)) {
            foreach ($goodsSkuList as $key => $value) {
                $list[$key]['goods_sku_id'] = $value['goods_sku_id'];
                $list[$key]['goods_spu_id'] = $value['goods_spu_id'];
                $list[$key]['life_account_id'] = $value['life_account_id'];
                $list[$key]['goods_name'] = $value['goods_name'];
                $list[$key]['life_account_name'] = !empty($lifeAccountList[$value['life_account_id']]['life_account_name']) ? $lifeAccountList[$value['life_account_id']]['life_account_name'] : '';  
            }
        }

        $marketing_info['first_commission_value'] = !empty($marketing_info['first_commission_value']) ? $marketing_info['first_commission_value']/100 : 0;
        $marketing_info['second_commission_value'] = !empty($marketing_info['second_commission_value']) ? $marketing_info['second_commission_value']/100 : 0;
        
        $info['marketing_info'] = $marketing_info;
        $info['goods_list'] = $list;
        return $info;
    }


    public static function getMarketingList($params)
    {
        $where['marketing_id'] = !empty($params['marketing_id']) ? $params['marketing_id'] : '';
        $marketing_list = Marketing::getMarketingList($where);
        return $marketing_list;
    }

   


   /**
    *判断长度
    *
    */
    public static function utf8Strlen($string = null) {
        // 将字符串分解为单元
        preg_match_all("/./us", $string, $match);
        // // 返回单元个数
        return count($match[0]);
    }
    
    /**
     * 通过发号器拿 id （非雪花）
     * number，获取店铺shop_id用生活号id后两位，商品相关的这里是shop_id的后两位
     * @param $number
     * @param $type
     * @param int $count
     * @return array|mixed
     */
    public static function getIdgenId($number, $type, $count = 1)
    {
        $res = Idgen::get(appConfig('idgen.partner'), appConfig('idgen.key'), [], [[
            "type" => $type,
            'number' => (int)$number,
            "count" => $count]]);
        return $res['id_datetime'][$type] ?? [];
    }

    public static function getSnowIdgenId($type, $count = 1)
    {
        $res = Idgen::get(appConfig('idgen.partner'), appConfig('idgen.key'), [[
            "type" => $type,
            "count" => $count]], []);
        return $res['id_snow'][$type] ?? [];
    }
}