<?php


namespace App\Services\marketing;

use Api\PhpUtils\Http\Request;
use App\Models\marketing\mysql\MarketingTakePlace;
use App\Models\marketing\mysql\TakePlace;
use App\Exception\custom\MarketingException;

class TakePlaceService
{
    const SELF_LIFE_ACCOUNT_ID = 123123;
    /**
     * 获取自提点列表（op 后台）
     *
     * 支持关键词（自提点名称/联系人）搜索
     * @param string $keywords
     * @param int $offset
     * @param int $limit
     * @return \Api\PhpUtils\Mysql\MysqlBase
     */
    public static function searchList($keywords='',$offset=0,$limit=20)
    {
        return TakePlace::searchList($keywords, $offset, $limit);
    }

    /**
     * 获取某个自提点详情
     * 有删除自提点的情况，订单获取自提点详情时忽略自提点状态直接按id获取
     * @param $takePlaceId
     * @return \Api\PhpUtils\Mysql\MysqlBase
     */
    public static function detail($takePlaceId)
    {
        return TakePlace::get('*',['take_place_id'=>$takePlaceId]);
    }

    /**
     * 添加自提点
     * @param $params
     * @return \Api\PhpUtils\Mysql\MysqlBase
     * @throws MarketingException
     */
    public static function add($params)
    {
        //验证life_account_id
        if(isset($params['life_account_id'])) {
            $url = config('interface', 'merchant.lifeaccount.get_life_account_by_id');
            $lifeAccountRes = (new Request())->get($url, ["life_account_id" => $params["life_account_id"]]);

            if (empty($lifeAccountRes["response"]["result"])) {
                throw new MarketingException(['cus' => MarketingException::LIFE_ACCOUNT_NO_EXIST]);
            }
        }

        $data['life_account_id'] = $params['life_account_id'] ?? self::SELF_LIFE_ACCOUNT_ID;
        $data['take_place_name'] = $params['name'];
        $data['longitude'] = $params['longitude'];
        $data['latitude'] = $params['latitude'];
        $data['address'] = $params['address'];
        $data['contact_name'] = $params['contact_name'] ?? '';
        $data['phone'] = $params['phone'] ?? '';
        $data['province'] = $params['province'];
        $data['city'] = $params['city'];
        $data['area'] = $params['area'];

        return TakePlace::insert($data);
    }

    /**
     * 删除自提点
     * @param $takePlaceId
     * @return \Api\PhpUtils\Mysql\MysqlBase
     */
    public static function delete($takePlaceId)
    {
        return TakePlace::update(
            ['is_delete'=>TakePlace::STATUS_DELETE],
            ['take_place_id'=>$takePlaceId]
        );
    }

    /**
     * 给拼单活动绑定自提点
     * @param $marketingPindanId
     * @param $takePlaceId
     * @return \Api\PhpUtils\Mysql\MysqlBase
     */
    public static function bindPindanTakePlace($marketingPindanId,$takePlaceId)
    {
        $data = [
            'marketing_pindan_id'=>$marketingPindanId,
            'take_place_id'=>$takePlaceId,
        ];

        //检查关联是否已存在
        if(!empty(MarketingTakePlace::get('*',$data))) {
            throw new MarketingException(['cus' => MarketingException::RELATION_ALREADY_EXIST]);
        }

        return MarketingTakePlace::insert($data);
    }

    /**
     * 获取拼单活动绑定的自提点列表
     * @param $marketingPindanId
     * @param int $offset
     * @param int $limit
     * @return array
     */
    public static function getPindanTakePlaceList($marketingPindanId,$offset=0,$limit=20)
    {
        $where = ['marketing_pindan_id' => $marketingPindanId];
        $where['ORDER'] = ['marketing_take_place_id' => 'DESC'];
        $where['LIMIT'] = [$offset, $limit];

//        $e = MarketingTakePlace::debug();
        $relations = MarketingTakePlace::select('*', $where);
        $makePlaceIds = array_column($relations,'take_place_id');

        $where = ['take_place_id'=>$makePlaceIds,'is_delete'=>TakePlace::STATUS_NORMAL];
        $takePlaces = TakePlace::select('*',$where);

        return $takePlaces ?: [];
    }

}